#ifdef _NES_MAPPER_CPP_

/////////////////////////////////////////////////////////////////////
// Mapper 243
void NES_mapper243::Reset()
{
  // set CPU bank pointers
  set_CPU_banks(0,1,2,3);

  // set PPU bank pointers
  if(num_1k_VROM_banks)
  {
    set_PPU_banks(0,1,2,3,4,5,6,7);
  }

  regs[0] = regs[1] = regs[2] = regs[3] = 0;
}

void NES_mapper243::MemoryWriteLow(uint32 addr, uint8 data)
{
  if(addr == 0x4100)
  {
    regs[0] = data;
  }
  else if(addr == 0x4101)
  {
    switch(regs[0] & 0x07)
    {
      case 0x02:
        {
          regs[1] = data & 0x01;
        }
        break;

      case 0x00:
      case 0x04:
      case 0x07:
        {
          regs[2] = (data & 0x01) << 1;
        }
        break;

      case 0x05:
        {
          set_CPU_bank4(data*4+0);
          set_CPU_bank5(data*4+1);
          set_CPU_bank6(data*4+2);
          set_CPU_bank7(data*4+3);
        }
        break;

      case 0x06:
        {
          regs[3] = (data & 0x03) << 2;
        }
        break;
    }
    if (num_1k_VROM_banks <= 64)
    {
      uint8 chr_bank = (regs[2] + regs[3]) >> 1;
      set_PPU_bank0(chr_bank*8+0);
      set_PPU_bank1(chr_bank*8+1);
      set_PPU_bank2(chr_bank*8+2);
      set_PPU_bank3(chr_bank*8+3);
      set_PPU_bank4(chr_bank*8+4);
      set_PPU_bank5(chr_bank*8+5);
      set_PPU_bank6(chr_bank*8+6);
      set_PPU_bank7(chr_bank*8+7);
    }
    else
    {
      uint8 chr_bank = regs[1] + regs[2] + regs[3];
      set_PPU_bank0(chr_bank*8+0);
      set_PPU_bank1(chr_bank*8+1);
      set_PPU_bank2(chr_bank*8+2);
      set_PPU_bank3(chr_bank*8+3);
      set_PPU_bank4(chr_bank*8+4);
      set_PPU_bank5(chr_bank*8+5);
      set_PPU_bank6(chr_bank*8+6);
      set_PPU_bank7(chr_bank*8+7);
    }
  }
}
/////////////////////////////////////////////////////////////////////

#endif
